from django.contrib import admin

from govuk_template_base.models import ServiceSettings, Link


@admin.register(Link)
class LinkAdmin(admin.ModelAdmin):
    list_display = ('name',)


@admin.register(ServiceSettings)
class ServiceSettingsAdmin(admin.ModelAdmin):
    list_display = ('name', 'phase')



import itertools

from django import template
from django.utils.translation import gettext_lazy as _

from govuk_template_base.models import ServiceSettings

register = template.Library()


@register.filter
def separate_thousands(value):
    if not isinstance(value, int):
        return value
    return '{:,}'.format(value)


@register.simple_tag
def get_service_settings():
    return ServiceSettings.default_settings()


@register.inclusion_tag('govuk_template_base/page-list.html')
def page_list(page, page_count, query_string=None, end_padding=1, page_padding=2):
    if page_count < 7:
        pages_with_ellipses = range(1, page_count + 1)
    else:
        pages = sorted(set(itertools.chain(
            range(1, end_padding + 2),
            range(page - page_padding, page + page_padding + 1),
            range(page_count - end_padding, page_count + 1),
        )))
        pages_with_ellipses = []
        last_page = 0
        for index in pages:
            if index < 1 or index > page_count:
                continue
            if last_page + 1 < index:
                pages_with_ellipses.append(None)
            pages_with_ellipses.append(index)
            last_page = index
    return {
        'page': page,
        'page_count': page_count,
        'page_range': pages_with_ellipses,
        'query_string': query_string,
    }


@register.inclusion_tag('govuk_template_base/pagination.html')
def pagination(prev_url, next_url, prev_title=None, next_title=None, prev=_('Previous'), next=_('Next')):
    context = {'prev': prev, 'next': next}
    if prev_url:
        context['prev_page'] = {'title': prev_title, 'url': prev_url}
    if next_url:
        context['next_page'] = {'title': next_title, 'url': next_url}
    return context



from django.db import migrations, models
import govuk_template_base.models


class Migration(migrations.Migration):
    initial = True
    dependencies = []
    operations = [
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('localise_name', models.BooleanField(default=False, verbose_name='Localise name')),
                ('link', models.CharField(max_length=255, verbose_name='Link')),
                ('link_is_view_name', models.BooleanField(default=False, verbose_name='Link is a view name')),
            ],
            options={
                'ordering': ('modified',),
                'verbose_name': 'Link',
                'verbose_name_plural': 'Links',
            },
        ),
        migrations.CreateModel(
            name='ServiceSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('localise_name', models.BooleanField(default=False, verbose_name='Localise name')),
                ('phase', models.CharField(choices=[('discovery', 'Discovery'), ('alpha', 'Alpha'), ('beta', 'Beta'), ('live', 'Live')], default='discovery', max_length=10, verbose_name='Phase')),
                ('header_link_view_name', models.CharField(blank=True, max_length=100, validators=[govuk_template_base.models.view_name_validator], verbose_name='View name')),
                ('footer_links', models.ManyToManyField(blank=True, related_name='_servicesettings_footer_links_+', to='govuk_template_base.Link', verbose_name='Footer links')),
                ('header_links', models.ManyToManyField(blank=True, related_name='_servicesettings_header_links_+', to='govuk_template_base.Link', verbose_name='Header links')),
            ],
            options={
                'ordering': ('-modified',),
                'verbose_name_plural': 'Service settings',
                'verbose_name': 'Service settings',
            },
        ),
    ]



import os
import threading
import warnings

from django.apps import apps
from django.conf import settings
from django.contrib.staticfiles.handlers import StaticFilesHandler
from django.core.management import CommandError, call_command
from django.core.management.commands.runserver import Command as RunserverCommand
try:
    from watchdog.events import FileSystemEventHandler
    from watchdog.observers import Observer
except ImportError:
    FileSystemEventHandler, Observer = object, None

base_options = ('verbosity', 'settings', 'pythonpath', 'traceback', 'no_color')


class Command(RunserverCommand, FileSystemEventHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.watched_apps = []
        self.build_delay = 1
        self.build_timer = None
        self.build_options = {}
        self.serve_static = False

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '--watch-app', nargs='*', dest='watched_apps',
            help='Watch source assets for changes and rebuild them.',
        )
        parser.add_argument(
            '--build-delay', default=self.build_delay, type=int,
            help='Time to delay rebuild (seconds).',
        )

    def handle(self, *args, **options):
        if not settings.DEBUG:
            raise CommandError('devserver is only available in DEBUG mode')
        if not options['use_threading']:
            raise CommandError('Threading is required')
        self.watched_apps = options.pop('watched_apps')
        if self.watched_apps:
            try:
                self.watched_apps = [apps.get_app_config(app_label) for app_label in self.watched_apps]
            except (LookupError, ImportError) as e:
                raise CommandError('%s. Are you sure your INSTALLED_APPS setting is correct?' % e)
        else:
            self.watched_apps = apps.get_app_configs()
        self.serve_static = 'django.contrib.staticfiles' in (a.name for a in apps.get_app_configs())
        self.build_delay = options['build_delay']
        self.build_options = {option: options[option] for option in base_options}
        if not Observer:
            warnings.warn('watchdog is not available, try installing using [watch] extra')
        elif not options['use_reloader'] or os.environ.get('RUN_MAIN'):  # try to only run one observer
            self.setup_observer()
        return super().handle(*args, **options)

    def setup_observer(self):
        observer = Observer()
        watched_app_paths = (os.path.join(app_config.path, 'static-src', 'stylesheets')
                             for app_config in self.watched_apps)
        watched_app_paths = filter(os.path.isdir, watched_app_paths)
        for watched_app_path in watched_app_paths:
            observer.schedule(self, watched_app_path, recursive=True)
        observer.daemon = True
        observer.start()

    def on_created(self, event):
        self.schedule_build(event.src_path)

    def on_modified(self, event):
        self.schedule_build(event.src_path)

    def schedule_build(self, path):
        if not path.lower().endswith('.scss'):
            return
        if self.build_options['verbosity']:
            self.stdout.write('Scheduling `buildscss`')
        if self.build_timer:
            self.build_timer.cancel()
        # TODO: only build apps with changed paths?
        self.build_timer = threading.Timer(self.build_delay, call_command,
                                           args=('buildscss',),
                                           kwargs=self.build_options)
        self.build_timer.start()

    def get_handler(self, *args, **options):
        handler = super().get_handler(*args, **options)
        if self.serve_static:
            handler = StaticFilesHandler(handler)
        return handler



import os
import textwrap

from django.apps import apps
from django.conf import settings
from django.core.management import BaseCommand, CommandError


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('args', metavar='app_labels', nargs='*',
                            help='Limit SCSS building to these apps.')
        parser.add_argument('--collect', action='store_true',
                            help='Save output CSS in collected static files location.')

    def handle(self, *app_labels, **options):
        if app_labels:
            try:
                app_configs = [apps.get_app_config(app_label) for app_label in app_labels]
            except (LookupError, ImportError) as e:
                raise CommandError('%s. Are you sure your INSTALLED_APPS setting is correct?' % e)
        else:
            app_configs = apps.get_app_configs()
        verbosity = options['verbosity']
        collected_dest = options['collect'] and os.path.join(settings.STATIC_ROOT, 'stylesheets')
        paths = (os.path.join(app_config.path, 'static-src', 'stylesheets') for app_config in app_configs)
        paths = list(filter(os.path.isdir, paths))
        for source_path in paths:
            dest_path = collected_dest or os.path.join(source_path, os.pardir, os.pardir, 'static', 'stylesheets')
            if verbosity > 1:
                self.stdout.write('Compiling SCSS files in %s' % source_path)
            compile_scss(source_path, dest_path, include_paths=paths)


def scss_defaults_importer(path):
    if path == 'govuk_template_base/defaults':
        static_url = getattr(settings, 'STATIC_URL', None) or '/static/'
        if not static_url.endswith('/'):
            static_url += '/'
        return [(path, textwrap.dedent('''
            // Image asset paths required by GOV.UK frontend toolkit functions
            $path: '%(static_url)simages/';
        ''' % dict(static_url=static_url)))]


def compile_scss(source_path, dest_path, include_paths=(), output_style='compressed'):
    try:
        import sass
    except ImportError:
        raise CommandError('libsass is not available, try installing using [scss] extra')
    sass.compile(dirname=(source_path, dest_path), include_paths=include_paths,
                 output_style=output_style,
                 importers=[(0, scss_defaults_importer)])



import enum

from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from django.db import models
from django.urls import NoReverseMatch, reverse
from django.utils.translation import gettext, gettext_lazy as _, pgettext_lazy


class ServicePhase(enum.Enum):
    discovery = pgettext_lazy('Service phase', 'Discovery')
    alpha = pgettext_lazy('Service phase', 'Alpha')
    beta = pgettext_lazy('Service phase', 'Beta')
    live = pgettext_lazy('Service phase', 'Live')

    @classmethod
    def model_choices(cls):
        return tuple((item.name, item.value) for item in cls)


def view_name_validator(view_name):
    try:
        reverse(view_name)
    except NoReverseMatch:
        raise ValidationError(_('View not found'), code='no_reverse_match')


class ServiceSettings(models.Model):
    modified = models.DateTimeField(auto_now=True)
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    localise_name = models.BooleanField(verbose_name=_('Localise name'), default=False)
    phase = models.CharField(verbose_name=_('Phase'), max_length=10,
                             choices=ServicePhase.model_choices(), default=ServicePhase.discovery.name)
    header_link_view_name = models.CharField(verbose_name=_('View name'), max_length=100,
                                             blank=True, validators=[view_name_validator])
    header_links = models.ManyToManyField('Link', verbose_name=_('Header links'), blank=True, related_name='+')
    footer_links = models.ManyToManyField('Link', verbose_name=_('Footer links'), blank=True, related_name='+')

    class Meta:
        ordering = ('-modified',)
        verbose_name = verbose_name_plural = _('Service settings')

    def __str__(self):
        return self.name

    @classmethod
    def default_settings(cls):
        return cls.objects.first() or cls.objects.create(name='Untitled')

    @property
    def localised_name(self):
        return gettext(self.name) if self.localise_name else self.name

    @property
    def header_link_url(self):
        if self.header_link_view_name:
            return reverse(self.header_link_view_name)
        return self.header_link_view_name

    @property
    def phase_name(self):
        return ServicePhase[self.phase].value

    @property
    def has_header_links(self):
        return self.header_links.exists()

    @property
    def has_footer_links(self):
        return self.footer_links.exists()


class Link(models.Model):
    modified = models.DateTimeField(auto_now=True)
    name = models.CharField(verbose_name=_('Name'), max_length=255)
    localise_name = models.BooleanField(verbose_name=_('Localise name'), default=False)
    link = models.CharField(verbose_name=_('Link'), max_length=255)
    link_is_view_name = models.BooleanField(verbose_name=_('Link is a view name'), default=False)

    class Meta:
        ordering = ('modified',)
        verbose_name = _('Link')
        verbose_name_plural = _('Links')

    def __str__(self):
        return self.name

    def clean(self):
        if self.link_is_view_name:
            view_name_validator(self.link)
        else:
            URLValidator(schemes=('http', 'https'))(self.link)

    @property
    def localised_name(self):
        return gettext(self.name) if self.localise_name else self.name

    @property
    def url(self):
        if self.link_is_view_name:
            return reverse(self.link)
        return self.link
